#!/QOpenSys/pkgs/bin/perl

use strict;

my $file = $ARGV[1];
my $new_path = $ARGV[2];

my ($file, $new_path) = @ARGV;

if (not defined $file or not defined $new_path)
{
    die "$0: [xcoff object] [path]\n";
}

my $new_path_len = length $new_path;

open my $fh, '+<:raw', $file;

# XCOFF format documented here: https://www.ibm.com/support/knowledgecenter/en/ssw_aix_72/com.ibm.aix.files/XCOFF.htm

# Read magic bytes and number of sections from the XCOFF header
my $bytes_read = read $fh, my $bytes, 4;
my ($f_magic, $f_nscns) = unpack 'nn', $bytes;

if ($f_magic != 0x01DF and $f_magic != 0x01F7)
{
    die sprintf("Doesn't seem to be an XCOFF object. Unknown magic value %X\n", $f_magic)
}

my $is64 = $f_magic == 0x01F7;

# Read the size of the auxiliary header so we can skip past it to the first section header
seek $fh, 16, 0;
$bytes_read = read $fh, $bytes, 2;

my ($f_opthdr) = unpack 'n', $bytes;

# size of the xcoff file header
my $file_header_size = $is64 ? 24 : 20;

my $offset = $file_header_size + $f_opthdr;
seek $fh, $offset, 0;

my $found = 0;
for(my $i = 0; $i < $f_nscns; ++$i)
{
    $bytes_read = read $fh, $bytes, 8;
    my $section = unpack 'Z8', $bytes;

    if ($section eq '.loader')
    {
        $found = 1;
        last;
    }
    
    $offset += $is64 ? 72 : 40;
    seek $fh, $offset, 0;
}

die "Couldn't find loader section\n" if (not $found);

# Read the offset to the loader section raw data
my $s_scnptr_offset = $is64 ? 32 : 20;
$offset += $s_scnptr_offset;
seek $fh, $offset, 0;
$bytes_read = read $fh, $bytes, ($is64 ? 8 : 4);

my $s_scnptr_fmt = ($is64 ? 'Q>' : 'N');
my $s_scnptr = unpack $s_scnptr_fmt, $bytes;

$offset = $s_scnptr;

# Read the import file id string table length and # of import file ids
seek $fh, $s_scnptr+12, 0;
$bytes_read = read $fh, $bytes, 8;
my ($l_istlen, $l_impid) = unpack 'NN', $bytes;

# Read the offset to the start of the import file ids
my $l_impoff_offset = $is64 ? 24 : 20;
$offset = $s_scnptr + $l_impoff_offset;
seek $fh, $offset, 0;
$bytes_read = read $fh, $bytes, $is64 ? 8 : 4;
my $l_impoff_fmt = ($is64 ? 'Q>' : 'N');
my $l_impoff = unpack $l_impoff_fmt, $bytes;

# Read the whole file import id string table
$offset = $s_scnptr + $l_impoff;
seek $fh, $offset, 0;
$bytes_read = read $fh, $bytes, $l_istlen;


# Unpack the PATH and calculate the difference in length
my $path = unpack "Z$l_istlen", $bytes;
my $path_len = length $path;
my $off = $path_len - $new_path_len;
die "You're PATH is longer than what's in the original file\n" if ($off < 0);

# Seek back to the start of the file id string table
seek $fh, $offset, 0;

# Write the new path, the rest of the original file id string table, and pad with NUL bytes
print $fh $new_path;
print $fh substr $bytes, $path_len;
print $fh "\x00" x $off;

close $fh;
